using System;
using Microsoft.SPOT;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Hardware;
using System.Threading;

namespace NetduinoSwitchingLights
{
    public class ActiveAnalog : IDisposable
    {
        private const int READ_INTERVAL = 20;
        private AnalogIn analogPin;
        private Thread readThread;
        private bool active = true;
        private int previousValue;
        private int currentValue;

        public delegate void AnalogValueChange(int currentValue, int offset);

        public event AnalogValueChange ValueChanged;

        public ActiveAnalog(FEZ_Pin.AnalogIn pin)
            : this(pin, 100)
        {
        }

        public ActiveAnalog(FEZ_Pin.AnalogIn pin, int maxScale)
        {
            if (maxScale < 1)
            {
                maxScale = 1;
            } else if (maxScale > 3300)
            {
                maxScale = 3300;
            }
            this.analogPin = new AnalogIn((AnalogIn.Pin)pin);
            this.analogPin.SetLinearScale(0, maxScale);
            this.readThread = new Thread(this.ReadValue);
            this.readThread.Start();
        }

        private void ReadValue()
        {
            this.currentValue = this.analogPin.Read();
            while (this.active)
            {
                this.previousValue = this.currentValue;
                this.currentValue = this.analogPin.Read();
                if (this.previousValue != this.currentValue)
                {
                    if (this.ValueChanged != null)
                    {
                        this.ValueChanged(this.currentValue, this.currentValue - this.previousValue);
                    }
                }
                try
                {
                    Thread.Sleep(READ_INTERVAL);
                }
                catch (Exception)
                {
                }
            }
        }

        public int CurrentValue
        {
            get
            {
                return this.currentValue;
            }
        }

        public int PreviousValue
        {
            get
            {
                return this.previousValue;
            }
        }

        public void Dispose()
        {
            this.active = false;
            this.readThread.Abort();
            this.readThread = null;
        }
    }
}
